"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var ts_smart_logger_1 = require("ts-smart-logger");
var util_1 = require("../../util");
var definition_1 = require("../../definition");
var application_interface_1 = require("./application.interface");
var application_action_builder_1 = require("./application-action.builder");
var universal_container_1 = require("../base/universal.container");
var UniversalApplicationContainer = /** @class */ (function (_super) {
    __extends(UniversalApplicationContainer, _super);
    /**
     * @stable - 23.04.2018
     * @param {TProps} props
     */
    function UniversalApplicationContainer(props) {
        var _this = _super.call(this, props) || this;
        _this.extraRoutes = new Map();
        _this.syncState = _this.syncState.bind(_this);
        _this.onBeforeLogout = _this.onBeforeLogout.bind(_this);
        _this.registerLogoutRoute();
        _this.registerStateTask();
        UniversalApplicationContainer.logger.debug('[$UniversalApplicationContainer][constructor] The app has been instantiated. The initial props are:', props);
        return _this;
    }
    /**
     * @stable [25.09.2019]
     */
    UniversalApplicationContainer.prototype.componentDidMount = function () {
        this.dispatchCustomType(application_action_builder_1.ApplicationActionBuilder.buildMountActionType());
        this.clearPreviousStates();
    };
    /**
     * @stable [24.09.2019]
     */
    UniversalApplicationContainer.prototype.componentWillUnmount = function () {
        this.unregisterStateTask();
        if (this.stateSettings.syncEnabled) {
            this.syncState(); // In case of "manual location.reload"
        }
    };
    /**
     * @stable [16.11.2019]
     * @param {RoutePredicateT} routePredicate
     * @returns {JSX.Element[]}
     */
    UniversalApplicationContainer.prototype.getRoutes = function (routePredicate) {
        if (routePredicate === void 0) { routePredicate = function () { return true; }; }
        return this.buildRoutes(this.dynamicRoutes, routePredicate).concat(this.buildRoutes(this.extraRoutes, routePredicate));
    };
    UniversalApplicationContainer.prototype.lookupConnectedContainerByRoutePath = function (path) {
        var _this = this;
        if (!path) {
            UniversalApplicationContainer.logger.warn('[$UniversalApplicationContainer][lookupConnectedContainerByRoutePath] The path is empty.');
            return null;
        }
        var result;
        this.dynamicRoutes.forEach(function (config, ctor) {
            var routeConfiguration = util_1.calc(config.routeConfiguration, _this.routes);
            if (routeConfiguration && [routeConfiguration.path, routeConfiguration.key].includes(path)) {
                result = ctor;
            }
        });
        if (!result) {
            UniversalApplicationContainer.logger.warn("[$UniversalApplicationContainer][lookupConnectedContainerByRoutePath] A component is not found by the path: " + path + ".");
        }
        return result;
    };
    UniversalApplicationContainer.prototype.registerRoute = function (container, config) {
        this.extraRoutes.set(container, config);
    };
    UniversalApplicationContainer.prototype.registerLogoutRoute = function () {
        var loginContainer = this.lookupConnectedContainerByRoutePath(this.routes.signIn);
        if (!loginContainer) {
            UniversalApplicationContainer.logger.debug('[$UniversalApplicationContainer][registerLogoutRoute] The login route is not registered.');
        }
        else {
            this.registerRoute(loginContainer, {
                routeConfiguration: {
                    type: definition_1.ContainerVisibilityTypesEnum.PUBLIC,
                    path: this.routes.logout,
                    beforeEnter: this.onBeforeLogout,
                    onEnter: this.onBeforeLogout,
                },
            });
            UniversalApplicationContainer.logger.debug('[$UniversalApplicationContainer][registerLogoutRoute] Logout router has been registered.');
        }
    };
    UniversalApplicationContainer.prototype.onBeforeLogout = function () {
        UniversalApplicationContainer.logger.debug('[$UniversalApplicationContainer][onBeforeLogout] Send a logout action.');
        this.dispatchCustomType(application_action_builder_1.ApplicationActionBuilder.buildLogoutActionType());
    };
    UniversalApplicationContainer.prototype.isMessageVisible = function () {
        var props = this.props;
        return props.progress || !!props.error || !props.ready;
    };
    UniversalApplicationContainer.prototype.getErrorMessage = function () {
        var _this = this;
        var props = this.props;
        return (util_1.orNull(props.error || props.customError, function () { return (props.customError
            ? props.error
            : _this.t(_this.settings.messages.followingErrorHasOccurredMessage) + " \"" + props.error.toLowerCase() + "\""); }));
    };
    /**
     * @stable [24.09.2019]
     */
    UniversalApplicationContainer.prototype.syncState = function () {
        try {
            this.doSyncState();
        }
        catch (e) {
            this.logManager.send(definition_1.ApplicationEventCategoriesEnum.STATE_ERROR, definition_1.ApplicationStateEventsEnum.SYNC, e.message);
        }
    };
    /**
     * @stable [24.09.2019]
     */
    UniversalApplicationContainer.prototype.doSyncState = function () {
        this.storage.set(definition_1.STORAGE_APP_STATE_KEY, this.stateSerializer.serialize(this.appStore.getState()));
    };
    /**
     * @stable [25.09.2019]
     */
    UniversalApplicationContainer.prototype.clearPreviousStates = function () {
        var _this = this;
        this.storage.each(function (value, key) {
            if (key.endsWith(definition_1.STORAGE_APP_STATE_KEY)) {
                _this.storage.remove(key, true);
            }
        });
    };
    /**
     * @stable [24.09.2019]
     */
    UniversalApplicationContainer.prototype.registerStateTask = function () {
        var _this = this;
        if (!this.stateSettings.syncEnabled) {
            return false;
        }
        if (this.stateSettings.syncTimeout > 0) {
            this.syncStateWithStorageTask = new util_1.DelayedTask(this.syncState.bind(this), this.stateSettings.syncTimeout);
            this.storeUnsubscriber = this.appStore.subscribe(function () { return _this.syncStateWithStorageTask.start(); });
        }
        return true;
    };
    /**
     * @stable [24.09.2019]
     */
    UniversalApplicationContainer.prototype.unregisterStateTask = function () {
        if (!this.stateSettings.syncEnabled) {
            return false;
        }
        if (!R.isNil(this.storeUnsubscriber)) {
            this.storeUnsubscriber();
            this.storeUnsubscriber = null;
        }
        if (!R.isNil(this.syncStateWithStorageTask)) {
            this.syncStateWithStorageTask.stop();
            this.syncStateWithStorageTask = null;
        }
        return true;
    };
    /**
     *
     * @param {Map<IContainerCtor, IConnectorEntity>} map
     * @param {RoutePredicateT} routePredicate
     * @returns {JSX.Element[]}
     */
    UniversalApplicationContainer.prototype.buildRoutes = function (map, routePredicate) {
        var _this = this;
        var routes0 = [];
        var routes = [];
        map.forEach(function (connectorEntity, containerCtor) {
            var routeCfg = connectorEntity.routeConfiguration;
            if (routePredicate.call(null, routeCfg)) {
                routes0.push(routeCfg.path || routeCfg.key);
                routes.push(_this.buildRoute(containerCtor, connectorEntity));
            }
        });
        UniversalApplicationContainer.logger.debug(function () { return "[$UniversalApplicationContainer][buildRoutes] The routes have been built. Routes: " + routes0.join('\n'); });
        return routes;
    };
    Object.defineProperty(UniversalApplicationContainer.prototype, "stateSettings", {
        /**
         * @stable [24.09.2019]
         * @returns {IStateSettings}
         */
        get: function () {
            return this.settings.state || {};
        },
        enumerable: true,
        configurable: true
    });
    UniversalApplicationContainer.defaultProps = {
        sectionName: application_interface_1.APPLICATION_SECTION,
    };
    UniversalApplicationContainer.logger = ts_smart_logger_1.LoggerFactory.makeLogger('UniversalApplicationContainer');
    return UniversalApplicationContainer;
}(universal_container_1.UniversalContainer));
exports.UniversalApplicationContainer = UniversalApplicationContainer;
//# sourceMappingURL=universal-application.container.js.map