import { IBaseEvent, IDomAccessor, IJQueryElement, IScrollConfigEntity, IXYEntity } from '../../definition';
export declare class DomAccessor implements IDomAccessor {
    private readonly environment;
    private readonly eventManager;
    private readonly settings;
    /**
     * @stable [30.10.2019]
     * @param {Element} element
     * @returns {boolean}
     */
    isElementFocused(element: Element): boolean;
    /**
     * @stable [28.10.2019]
     * @param {(e: IBaseEvent) => void} callback
     * @returns {() => void}
     */
    attachClickListenerToDocument(callback: (e: IBaseEvent) => void): () => void;
    /**
     * @stable [26.10.2019]
     * @param {Element} child
     * @param {Element} parent
     * @returns {boolean}
     */
    isElementVisibleWithinParent(child: Element, parent: Element): boolean;
    /**
     * @stable [22.10.2019]
     * @param {Element} target
     * @param {string} classNames
     * @returns {boolean}
     */
    hasClasses(target: Element, ...classNames: string[]): boolean;
    /**
     * @stable [16.10.2019]
     * @param {string | Element} selector
     * @param {Element} parent
     * @returns {Element}
     */
    findElement(selector: string | Element, parent?: Element): Element;
    /**
     * @stable [08.10.2019]
     * @returns {boolean}
     */
    isAlreadyFocused(): boolean;
    /**
     * @stable [07.10.2019]
     * @param {string} path
     */
    redirect(path: string): void;
    /**
     * @stable [16.10.2019]
     * @param {boolean} forceReload
     */
    reload(forceReload?: boolean): void;
    /**
     * @stable [01.10.2019]
     * @returns {Element}
     */
    addRootElement(): Element;
    /**
     * @stable [30.09.2019]
     * @param {(e: Error) => void} callback
     */
    defineGlobalErrorHandler(callback: (e: Error) => void): void;
    /**
     * @stable [30.09.2019]
     * @param {string} tag
     * @param {Element} parent
     * @returns {Element}
     */
    createElement<TElement extends HTMLElement = HTMLElement>(tag?: string, parent?: Element): TElement;
    /**
     * @stable [30.09.2019]
     * @param {Element} child
     * @param {Element} parent
     * @returns {Element}
     */
    addChild(child: Element, parent?: Element): Element;
    /**
     * @stable [14.04.2019]
     * @param {Element} child
     * @param {Element} parent
     */
    removeChild(child: Element, parent?: Element): void;
    /**
     * @stable [04.04.2019]
     * @param {HTMLElement} element
     */
    enableFullScreen(element?: HTMLElement): void;
    /**
     * @stable [04.04.2019]
     * @param {HTMLElement} element
     */
    disableFullScreen(element?: HTMLElement): void;
    /**
     * @stable [16.02.2019]
     * @param {string | Element} selector
     * @param {Element} target
     * @returns {boolean}
     */
    hasElements(selector: string | Element, target: Element): boolean;
    /**
     * @stable [16.02.2019]
     * @param {string} selector
     * @param {Element} target
     * @returns {boolean}
     */
    hasParent(selector: string, target: Element): boolean;
    /**
     * @stable [29.09.2019]
     * @returns {Element}
     */
    getRootElement(): Element;
    /**
     * @stable [13.01.2019]
     * @returns {Element}
     */
    getDocumentBodyElement(): Element;
    /**
     * @stable [13.01.2019]
     * @param {Element} element
     * @param {string} clsNames
     */
    addClassNames(element: Element, ...clsNames: string[]): void;
    /**
     * @stable [13.01.2019]
     * @param {Element} element
     * @param {string} clsNames
     */
    removeClassNames(element: Element, ...clsNames: string[]): void;
    /**
     * @stable [18.12.2018]
     * @param {Element} el
     * @param {number} left
     */
    setScrollLeft(el: Element, left: number): void;
    /**
     * @stable [18.12.2018]
     * @param {Element} el
     * @returns {number}
     */
    getScrollLeft(el: Element): number;
    /**
     * @stable [18.12.2018]
     * @param {Element} el
     * @returns {number}
     */
    getScrollTop(el: Element): number;
    /**
     * @stable [13.12.2018]
     * @param {Element} source
     * @returns {number}
     */
    getHeight(source: Element): number;
    /**
     * @stable [13.12.2018]
     * @param {Element} source
     * @returns {IJQueryElement}
     */
    toJqEl(source: Element): IJQueryElement;
    /**
     * @stable [01.12.2018]
     * @param {Element} el
     * @returns {IXYEntity}
     */
    getScrollInfo(el: Element): IXYEntity;
    /**
     * @stable [01.12.2018]
     * @param {Element} source
     * @returns {number}
     */
    getContentHeight(source: Element): number;
    /**
     * @stable [08.11.2019]
     * @param {IXYEntity | Element} payload
     * @param {Element} parent
     * @param {IScrollConfigEntity} config
     */
    scrollTo(payload: IXYEntity | Element, parent?: Element, config?: IScrollConfigEntity): void;
    /**
     * @stable [01.12.2018]
     * @param {Element} el
     * @param {number} top
     */
    setScrollTop(el: Element, top: number): void;
    /**
     * @stable [09.10.2019]
     * @returns {Element}
     */
    getActiveElement(): Element;
    /**
     * @stable [17.10.2019]
     * @param {string} id
     * @returns {Element}
     */
    getElement(id: string): Element;
    /**
     * @stable [01.10.2019]
     * @returns {IBootstrapSettings}
     */
    private readonly bootstrapSettings;
    /**
     * @stable [08.10.2019]
     * @returns {Document}
     */
    private readonly document;
    /**
     * @stable [08.10.2019]
     * @returns {Window}
     */
    private readonly window;
    /**
     * @stable [09.10.2019]
     * @param {Element} element
     * @returns {boolean}
     */
    private isInputElement;
}
