"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var R = require("ramda");
var util_1 = require("../../../util");
var definitions_interface_1 = require("../../../definitions.interface");
var universal_field_component_1 = require("./universal-field.component");
var definition_1 = require("../../../definition");
var Field = /** @class */ (function (_super) {
    __extends(Field, _super);
    /**
     * @stable [28.10.2019]
     * @param {TProps} props
     */
    function Field(props) {
        var _this = _super.call(this, props) || this;
        _this.defaultActions = [];
        _this.inputRef = React.createRef();
        _this.onClick = _this.onClick.bind(_this);
        return _this;
    }
    /**
     * @stable [26.05.2019]
     * @returns {JSX.Element}
     */
    Field.prototype.render = function () {
        return (React.createElement("div", { className: this.getFieldClassName(), onClick: util_1.cancelEvent },
            this.props.children,
            this.getSelfElement(),
            this.getMessageElement(),
            this.getErrorMessageElement(),
            this.getAttachmentElement(),
            this.getKeyboardElement()));
    };
    /**
     * @stable [29.10.2019]
     */
    Field.prototype.componentWillUnmount = function () {
        _super.prototype.componentWillUnmount.call(this);
        this.onCloseVirtualKeyboard();
    };
    /**
     * @stable [07.01.2019]
     * @param {AnyT} currentRawValue
     */
    Field.prototype.onChangeManually = function (currentRawValue) {
        this.updateInputBeforeHTML5Validation(currentRawValue);
        _super.prototype.onChangeManually.call(this, currentRawValue);
    };
    /**
     * @stable [18.06.2018]
     * @param {IChangeEvent} event
     * @returns {AnyT}
     */
    Field.prototype.getRawValueFromEvent = function (event) {
        return event.target.value;
    };
    /**
     * @stable [17.06.2018]
     */
    Field.prototype.resetError = function () {
        if (!R.isNil(this.input.required)) {
            // Because of Flux-architecture (chicken-and-egg dilemma)
            // This flag is being set by a redux engine on a new cycle
            // Need to reset: required, pattern, etc...
            this.input.required = false;
        }
        _super.prototype.resetError.call(this);
    };
    /**
     * @stable [03.09.2018]
     * @param {IKeyboardEvent} event
     */
    Field.prototype.onKeyDown = function (event) {
        var key = event.key;
        switch (key) {
            case 'Tab':
                this.onKeyTab(event);
                break;
            case 'Enter':
                this.onKeyEnter(event);
                break;
            case 'Escape':
                this.onKeyEscape(event);
                break;
            case 'ArrowDown':
                this.onKeyArrowDown(event);
                break;
            case 'ArrowUp':
                this.onKeyArrowUp(event);
                break;
            case 'Backspace':
                this.onKeyBackspace(event);
                break;
        }
        _super.prototype.onKeyDown.call(this, event);
    };
    /**
     * @stable [29.10.2019]
     */
    Field.prototype.setFocus = function () {
        var _this = this;
        util_1.ifNotNilThanValue(this.input, function (input) {
            if (!_this.isFocusPrevented) {
                input.focus();
            }
        });
    };
    Object.defineProperty(Field.prototype, "input", {
        /**
         * @stable [28.10.2019]
         * @returns {InputElementT}
         */
        get: function () {
            return util_1.ifNotNilThanValue(this.inputRef.current, function (input) { return input.inputElement || input; });
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [29.10.2019]
     * @returns {JSX.Element}
     */
    Field.prototype.getKeyboardElement = function () {
        var _this = this;
        return util_1.orNull(this.isKeyboardUsed && this.isKeyboardOpen(), function () { return _this.keyboardElement; });
    };
    Object.defineProperty(Field.prototype, "keyboardElement", {
        /**
         * @stable [29.10.2019]
         * @returns {JSX.Element}
         */
        get: function () {
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Field.prototype.getSelfElement = function () {
        var _this = this;
        var props = this.props;
        return util_1.orNull(props.fieldRendered !== false, function () { return (React.createElement("div", { ref: 'self', style: props.style, className: _this.getSelfElementClassName() },
            _this.getPrefixLabelElement(),
            _this.getInputWrapperElement(),
            _this.actionsElement,
            _this.getLabelElement(),
            _this.getProgressLabelElement())); });
    };
    Field.prototype.getInputWrapperElement = function () {
        return (React.createElement("div", { className: this.getInputWrapperElementClassName() },
            this.getInputElement(),
            this.getMirrorInputElement(),
            this.getInputCaretElement(),
            this.getInputAttachmentElement()));
    };
    Field.prototype.getProgressLabelElement = function () {
        return null;
    };
    Object.defineProperty(Field.prototype, "actionsElement", {
        get: function () {
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Field.prototype.getLabelElement = function () {
        var _this = this;
        var props = this.props;
        return util_1.ifNotNilThanValue(props.label, function () { return (React.createElement("label", { className: 'rac-field-label' }, _this.t(props.label))); });
    };
    Field.prototype.getMirrorInputElement = function () {
        return null;
    };
    Field.prototype.getInputCaretElement = function () {
        return null;
    };
    Field.prototype.getPrefixLabelElement = function () {
        return null;
    };
    Field.prototype.getAttachmentElement = function () {
        return null;
    };
    Object.defineProperty(Field.prototype, "hasInput", {
        /**
         * @stable [28.10.2019]
         * @returns {boolean}
         */
        get: function () {
            return !R.isNil(this.inputRef.current);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [20.08.2018]
     */
    Field.prototype.removeFocus = function () {
        var input = this.input;
        if (!R.isNil(input)) {
            input.blur();
        }
    };
    /**
     * @stable [10.11.2019]
     * @param {IBaseEvent} event
     */
    Field.prototype.onClick = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onClick)) {
            props.onClick(event);
        }
    };
    /**
     * @stable [31.08.2018]
     * @returns {JSX.Element}
     */
    Field.prototype.getInputElement = function () {
        return React.createElement("input", __assign({}, this.getInputElementProps()));
    };
    /**
     * @stable [30.10.2019]
     * @returns {IFieldComplexInputAttributes}
     */
    Field.prototype.getInputElementProps = function () {
        var _this = this;
        var props = this.props;
        /**/
        var autoComplete = props.autoComplete || 'off'; /* @stable [29.10.2019] */
        var cols = props.cols; /* @stable [28.10.2019] */
        var disabled = this.isDisabled; /* @stable [28.10.2019] */
        var maxLength = props.maxLength; /* @stable [28.10.2019] */
        var minLength = props.minLength; /* @stable [28.10.2019] */
        var name = props.name; /* @stable [28.10.2019] */
        var pattern = this.getFieldPattern(); /* @stable [29.10.2019] */
        var placeholder = util_1.orUndef(props.placeholder && !this.inProgress, function () { return _this.t(props.placeholder); }); /* @stable [29.10.2019] */
        var readOnly = this.isInactive; /* @stable [28.10.2019] */
        var required = this.isRequired; /* @stable [29.10.2019] */
        var rows = props.rows; /* @stable [28.10.2019] */
        var step = props.step; /* @stable [28.10.2019] */
        var tabIndex = props.tabIndex; /* @stable [28.10.2019] */
        var type = props.type || 'text'; /* @stable [28.10.2019] */
        var value = this.displayValue; /* @stable [28.10.2019] */
        var result = util_1.defValuesFilter(__assign({ className: 'rac-field-input rac-flex-full', autoComplete: autoComplete, cols: cols, disabled: disabled, maxLength: maxLength, minLength: minLength, name: name, pattern: pattern,
            placeholder: placeholder, readOnly: readOnly, rows: rows, step: step, tabIndex: tabIndex, type: type, value: value, required: required }, (this.isActive
            ? {
                onBlur: this.onBlur,
                onChange: this.onChange,
                onClick: this.onClick,
                onFocus: this.onFocus,
                onKeyDown: this.onKeyDown,
                onKeyUp: this.onKeyUp,
            }
            : {})));
        return __assign({}, result, { ref: this.inputRef });
    };
    /**
     * @stable [06.10.2018]
     * @returns {string}
     */
    Field.prototype.getFieldClassName = function () {
        var props = this.props;
        var _a = this.settings.bootstrap.flexEnabled, flexEnabled = _a === void 0 ? false : _a;
        return util_1.joinClassName('rac-field', flexEnabled && util_1.fullFlexClassName(props), this.isRequired && 'rac-field-required', this.isFieldInvalid() && 'rac-field-invalid', this.isValuePresent ? 'rac-field-value-present' : 'rac-field-value-not-present', this.isChangeable ? 'rac-field-changeable' : 'rac-field-not-changeable', this.isFocused ? 'rac-field-focused' : 'rac-field-not-focused', this.isDisabled && 'rac-field-disabled', this.isReadOnly && 'rac-field-readonly', this.isFocusPrevented && 'rac-field-prevent-focus', !R.isEmpty(this.getFieldActions()) && 'rac-field-actioned', props.label && 'rac-field-labeled', props.prefixLabel ? 'rac-field-label-prefixed' : 'rac-field-label-not-prefixed', props.className);
    };
    /**
     * @stable [23.10.2019]
     * @returns {string}
     */
    Field.prototype.getInputWrapperElementClassName = function () {
        return 'rac-field-input-wrapper';
    };
    /**
     * @stable [24.10.2019]
     * @returns {string}
     */
    Field.prototype.getSelfElementClassName = function () {
        // A field may be invisible, but attachment - visible
        return util_1.joinClassName('rac-self-field', !this.isVisible && 'rac-invisible');
    };
    /**
     * @stable [31.07.2018]
     * @returns {boolean}
     */
    Field.prototype.isInputValid = function () {
        return !this.hasInput || this.input.validity.valid;
    };
    /**
     * @stable [31.07.2018]
     * @returns {string}
     */
    Field.prototype.getNativeInputValidationMessage = function () {
        return this.input.validationMessage;
    };
    /**
     * @stable [31.07.2018]
     * @param {string} error
     */
    Field.prototype.setNativeInputValidity = function (error) {
        if (this.hasInput) {
            this.input.setCustomValidity(error);
        }
    };
    /**
     * @stable [03.09.2018]
     * @param {string} message
     * @param {string} className
     * @returns {JSX.Element}
     */
    Field.prototype.toMessageElement = function (message, className) {
        return (React.createElement("div", { title: message, className: util_1.toClassName('rac-field-help-text', className) }, message ? this.t(message) : definitions_interface_1.UNI_CODES.noBreakSpace));
    };
    /**
     * @stable [30.10.2019]
     * @returns {IFieldActionEntity[]}
     */
    Field.prototype.getFieldActions = function () {
        var props = this.props;
        var defaultActions = this.defaultActions || [];
        var actions = props.actions || [];
        return props.actionsPosition === definition_1.FieldActionPositionsEnum.LEFT
            ? defaultActions.concat(actions)
            : actions.concat(defaultActions);
    };
    /**
     * @stable [17.06.2018]
     * @param {AnyT} value
     */
    Field.prototype.updateInputBeforeHTML5Validation = function (value) {
        if (this.hasInput) {
            // We must update the field manually before calls HTML5 validation
            this.input.value = value;
        }
    };
    Object.defineProperty(Field.prototype, "jqInput", {
        /**
         * @stable [29.09.2019]
         * @returns {IJQueryElement}
         */
        get: function () {
            return this.domAccessor.toJqEl(this.input);
        },
        enumerable: true,
        configurable: true
    });
    return Field;
}(universal_field_component_1.UniversalField));
exports.Field = Field;
//# sourceMappingURL=field.component.js.map