/// <reference types="react" />
import { ILogger } from 'ts-smart-logger';
import { IUniversalField } from '../../../entities-definitions.interface';
import { IUniversalFieldProps } from '../../../configurations-definitions.interface';
import { AnyT, IFocusEvent, IKeyboardEvent } from '../../../definitions.interface';
import { IUniversalFieldState } from './field.interface';
import { UniversalComponent } from '../../base/universal.component';
export declare abstract class UniversalField<TProps extends IUniversalFieldProps, TState extends IUniversalFieldState> extends UniversalComponent<TProps, TState> implements IUniversalField<TProps, TState> {
    protected static logger: ILogger;
    private static DEFAULT_CARET_BLINKING_FREQUENCY_TIMEOUT;
    private caretBlinkingTask;
    /**
     * @stable [17.06.2018]
     * @param {TProps} props
     */
    constructor(props: TProps);
    /**
     * @stable [20.08.2018]
     */
    componentDidMount(): void;
    /**
     * @stable [04.09.2018]
     * @param {TProps} prevProps
     * @param {TState} prevState
     */
    componentDidUpdate(prevProps: TProps, prevState: TState): void;
    /**
     * @stable [17.06.2018]
     */
    resetError(): void;
    /**
     * @stable [20.08.2018]
     */
    clearValue(): void;
    /**
     * @stable [27.07.2018]
     * @param {AnyT} event
     */
    onChange(event: AnyT): void;
    /**
     * @stable [07.01.2018]
     * @param {AnyT} currentRawValue
     */
    onChangeManually(currentRawValue: AnyT): void;
    /**
     * @stable [22.10.2018]
     * @returns {AnyT}
     */
    readonly value: AnyT;
    /**
     * @stable [18.06.2018]
     * @param {IKeyboardEvent} event
     */
    onKeyBackspace(event: IKeyboardEvent): void;
    /**
     * @stable [18.06.2018]
     * @param {IKeyboardEvent} event
     */
    onKeyUp(event: IKeyboardEvent): void;
    /**
     * @stable [03.09.2018]
     * @param {IKeyboardEvent} event
     */
    onKeyDown(event: IKeyboardEvent): void;
    /**
     * @stable [18.06.2018]
     * @param {IKeyboardEvent} event
     */
    onKeyEnter(event: IKeyboardEvent): void;
    /**
     * @stable [04.09.2018]
     * @param {IKeyboardEvent} event
     */
    onKeyTab(event: IKeyboardEvent): void;
    /**
     * @stable [18.06.2018]
     * @param {IKeyboardEvent} event
     */
    onKeyEscape(event: IKeyboardEvent): void;
    /**
     * @stable [18.06.2018]
     * @param {IKeyboardEvent} event
     */
    onKeyArrowDown(event: IKeyboardEvent): void;
    /**
     * @stable [18.06.2018]
     * @param {IKeyboardEvent} event
     */
    onKeyArrowUp(event: IKeyboardEvent): void;
    /**
     * @stable [17.06.2018]
     * @param {AnyT} event
     * @returns {AnyT}
     */
    abstract getRawValueFromEvent(event: AnyT): AnyT;
    /**
     * @stable [06.06.2018]
     */
    abstract setFocus(): void;
    /**
     * @stable [29.10.2019]
     * @param {boolean} usePrintf
     * @param {AnyT} args
     * @returns {string}
     */
    protected buildDisplayMessage(usePrintf: boolean, ...args: AnyT[]): string;
    /**
     * @stable [30.10.2019]
     * @param {AnyT} value
     * @returns {boolean}
     */
    protected readonly isValuePresent: boolean;
    /**
     * @stable [30.10.2019]
     * @returns {boolean}
     */
    protected readonly isValueNotPresent: boolean;
    /**
     * @stable [29.10.2019]
     * @returns {boolean}
     */
    protected readonly isRequired: boolean;
    /**
     * @stable [05.10.2018]
     * @returns {boolean}
     */
    protected isFieldInvalid(): boolean;
    /**
     * @stable [17.06.2018]
     * @returns {AnyT}
     */
    protected getEmptyValue(): AnyT;
    /**
     * @stable [18.06.2018]
     * @returns {AnyT}
     */
    protected readonly definiteValue: AnyT;
    /**
     * @stable [31.07.2018]
     * @param {AnyT} value
     * @returns {string}
     */
    protected validateValueAndSetCustomValidity(value: AnyT): string;
    /**
     * @stable [31.07.2018]
     * @param {AnyT} value
     * @returns {string}
     */
    protected validateValue(value: AnyT): string;
    /**
     * @stable [03.09.2018]
     * @returns {boolean}
     */
    protected isInputValid(): boolean;
    /**
     * @stable [31.07.2018]
     * @returns {string}
     */
    protected getNativeInputValidationMessage(): string;
    /**
     * @stable [31.07.2018]
     * @param {string} error
     */
    protected setNativeInputValidity(error: string): void;
    /**
     * @stable [17.06.2018]
     * @returns {Array<string | RegExp>}
     */
    protected getFieldMask(): Array<string | RegExp>;
    /**
     * @stable [17.06.2018]
     * @returns {string}
     */
    protected getFieldPattern(): string;
    /**
     * @stable [18.06.2018]
     * @returns {AnyT}
     */
    protected readonly displayValue: AnyT;
    /**
     * @stable [12.02.2019]
     * @param {AnyT} value
     * @param {boolean} returnDisplayValue
     * @returns {AnyT}
     */
    protected getDecoratedValue(value: AnyT, returnDisplayValue?: boolean): AnyT;
    /**
     * @stable [07.01.2018]
     * @returns {boolean}
     */
    protected readonly hasDisplayValue: boolean;
    /**
     * @stable [07.01.2018]
     * @param {AnyT} value
     * @returns {AnyT}
     */
    protected decorateValueBeforeDisplaying(value: AnyT): AnyT;
    /**
     * @stable [28.10.2019]
     * @returns {boolean}
     */
    protected readonly isInactive: boolean;
    /**
     * @stable [28.10.2019]
     * @returns {boolean}
     */
    protected readonly isActive: boolean;
    /**
     * @stable [28.10.2019]
     * @returns {boolean}
     */
    protected readonly isDisabled: boolean;
    /**
     * @stable [29.10.2019]
     * @returns {boolean}
     */
    protected readonly isReadOnly: boolean;
    /**
     * @stable [28.10.2019]
     * @returns {boolean}
     */
    protected readonly isChangeable: boolean;
    /**
     * @stable [30.10.2019]
     * @returns {boolean}
     */
    protected readonly isFieldNotModifiable: boolean;
    /**
     * @stable [30.10.2019]
     * @returns {boolean}
     */
    protected readonly isFocused: boolean;
    /**
     * @stable [25.10.2019]
     * @returns {boolean}
     */
    protected readonly isVisible: boolean;
    /**
     * @stable [25.10.2019]
     * @returns {boolean}
     */
    protected readonly inProgress: boolean;
    /**
     * @stable [30.10.2019]
     * @param {IFocusEvent} event
     */
    protected onFocus(event: IFocusEvent): void;
    /**
     * @stable [30.10.2019]
     * @param {IFocusEvent} event
     */
    protected onBlur(event: IFocusEvent): void;
    /**
     * @stable [03.09.2018]
     * @returns {JSX.Element}
     */
    protected getInputAttachmentElement(): JSX.Element;
    /**
     * @stable [28.10.2019]
     * @returns {boolean}
     */
    protected openVirtualKeyboard(): boolean;
    /**
     * @stable [28.10.2019]
     */
    protected closeVirtualKeyboard(): void;
    /**
     * @stable [28.10.2019]
     */
    protected onCloseVirtualKeyboard(): void;
    /**
     * @stable [28.10.2019]
     * @returns {boolean}
     */
    protected isKeyboardOpen(): boolean;
    /**
     * @stable [24.10.2019]
     * @returns {boolean}
     */
    protected readonly isFocusPrevented: boolean;
    /**
     * @stable [24.10.2019]
     * @returns {boolean}
     */
    protected readonly isKeyboardUsed: boolean;
    /**
     * @stable [03.09.2018]
     * @returns {JSX.Element}
     */
    protected getErrorMessageElement(): JSX.Element;
    /**
     * @stable [03.09.2018]
     * @returns {JSX.Element}
     */
    protected getMessageElement(): JSX.Element;
    /**
     * @stable [03.09.2018]
     * @returns {string}
     */
    protected readonly error: string;
    /**
     * @stable [04.09.2018]
     */
    protected refreshCaretPosition(): void;
    /**
     * @stable [04.09.2018]
     * @returns {number}
     */
    protected getCaretPosition(): number;
    /**
     * @stable [30.10.2019]
     * @returns {boolean}
     */
    protected readonly isSyntheticCursorUsed: boolean;
    /**
     * @stable [03.09.2018]
     */
    protected abstract removeFocus(): void;
    /**
     * @stable [22.03.2019]
     * @param {string} message
     * @param {string} className
     * @returns {JSX.Element}
     */
    protected abstract toMessageElement(message: string, className?: string): JSX.Element;
    /**
     * @stable [31.07.2018]
     * @param {AnyT} currentRawValue
     */
    private onChangeValue;
    /**
     * @stable [31.07.2018]
     * @param {AnyT} rawValue
     */
    private validateField;
    /**
     * @stable [16.11.2019]
     * @param {AnyT} rawValue
     */
    private notifyFormAboutChanges;
    /**
     * @stable [16.11.2019]
     * @param {AnyT} rawValue
     */
    private notifyAboutChanges;
    /**
     * @stable [09.11.2019]
     */
    private setCaretVisibility;
}
