"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var R = require("ramda");
var definitions_interface_1 = require("../../definitions.interface");
var util_1 = require("../../util");
var base_1 = require("../base");
var dialog_1 = require("../dialog");
var layout_1 = require("../layout");
var field_1 = require("../field");
var list_1 = require("../list");
var Menu = /** @class */ (function (_super) {
    __extends(Menu, _super);
    /**
     * @stable [31.07.2018]
     * @param {IMenuProps} props
     */
    function Menu(props) {
        var _this = _super.call(this, props) || this;
        _this.dialogRef = React.createRef();
        _this.fieldRef = React.createRef();
        _this.menuAnchorRef = React.createRef();
        /**
         * @stable [17.06.2019]
         * @param {IMenuItemEntity} itm
         * @returns {StringNumberT}
         */
        _this.optionValueFn = function (itm) { return (itm.label ? _this.t(itm.label) : itm.value); };
        _this.hide = _this.hide.bind(_this);
        _this.onSelect = _this.onSelect.bind(_this);
        _this.onInputChange = _this.onInputChange.bind(_this);
        _this.onCloseAction = _this.onCloseAction.bind(_this);
        _this.onDialogDeactivate = _this.onDialogDeactivate.bind(_this);
        _this.getItemElement = _this.getItemElement.bind(_this);
        _this.state = { opened: false };
        _this.filterQueryTask = new util_1.DelayedTask(_this.onFilterChange.bind(_this), 500);
        return _this;
    }
    /**
     * @stable [18.06.2019]
     * @returns {JSX.Element}
     */
    Menu.prototype.render = function () {
        var props = this.props;
        var filterElement = this.filterElement;
        if (props.centeredMenu) {
            return (this.state.opened && (React.createElement(dialog_1.Dialog, { ref: this.dialogRef, className: 'rac-menu-dialog', closable: false, acceptable: false, onDeactivate: this.onDialogDeactivate },
                this.closeActionElement,
                filterElement,
                this.listElement)));
        }
        return (React.createElement("div", { ref: this.menuAnchorRef, className: this.uiFactory.menuAnchor },
            React.createElement("div", { ref: this.selfRef, style: __assign({}, !props.centeredMenu && { width: util_1.calc(props.width) }), className: util_1.joinClassName('rac-menu', props.className, this.uiFactory.menu, this.uiFactory.menuSurface) },
                filterElement,
                this.listElement)));
    };
    /**
     * @stable [18.06.2019]
     */
    Menu.prototype.componentWillUnmount = function () {
        _super.prototype.componentWillUnmount.call(this);
        this.filterQueryTask.stop();
    };
    Object.defineProperty(Menu.prototype, "centeredMenu", {
        /**
         * @stable [17.06.2019]
         * @returns {boolean}
         */
        get: function () {
            return this.props.centeredMenu;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [04.10.2018]
     */
    Menu.prototype.show = function () {
        var _this = this;
        var props = this.props;
        this.setState({ filter: definitions_interface_1.UNDEF, opened: true }, function () {
            if (_this.doesDialogExist) {
                _this.dialog.activate();
            }
            if (props.useFilter) {
                _this.field.setFocus();
            }
        });
        if (!R.isNil(util_1.nvl(props.xPosition, props.yPosition))) {
            util_1.setAbsoluteOffsetByCoordinates(this.menuAnchorRef.current, props.xPosition, props.yPosition);
        }
    };
    /**
     * @stable [18.06.2019]
     */
    Menu.prototype.hide = function () {
        var props = this.props;
        this.setState({ opened: false });
        if (util_1.isFn(props.onClose)) {
            props.onClose();
        }
    };
    /**
     * @stable [29.11.2018]
     * @returns {boolean}
     */
    Menu.prototype.isOpen = function () {
        return this.state.opened;
    };
    Object.defineProperty(Menu.prototype, "menuItems", {
        /**
         * @stable [06.08.2018]
         * @returns {IMenuItemEntity[]}
         */
        get: function () {
            var props = this.props;
            var state = this.state;
            var query = state.filter;
            return props.useFilter && props.remoteFilter !== true
                ? props.options.filter(function (option) { return props.filter(query, option); })
                : props.options;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [17.10.2018]
     * @param {IBaseEvent} event
     */
    Menu.prototype.onCloseAction = function (event) {
        util_1.cancelEvent(event);
        this.hide();
    };
    /**
     * @stable [04.10.2018]
     * @param {string} filter
     */
    Menu.prototype.onInputChange = function (filter) {
        this.setState({ filter: filter });
        this.filterQueryTask.start();
    };
    /**
     * @stable [17.06.2019]
     */
    Menu.prototype.onFilterChange = function () {
        var props = this.props;
        var state = this.state;
        if (util_1.isFn(props.onFilterChange)) {
            props.onFilterChange(state.filter);
        }
    };
    /**
     * @stable [06.08.2018]
     * @param {IMenuItemEntity} option
     */
    Menu.prototype.onSelect = function (option) {
        var props = this.props;
        if (props.onSelect) {
            props.onSelect(option);
        }
        if (!props.multi
            || this.menuItems.length === 1 // Because a "Flux Cycle", to prevent empty list after updating
        ) {
            this.hide();
        }
    };
    /**
     * @stable [17.06.2019]
     */
    Menu.prototype.onDialogDeactivate = function () {
        this.setState({ opened: false });
    };
    Object.defineProperty(Menu.prototype, "listElement", {
        /**
         * @stable [23.09.2019]
         * @returns {JSX.Element}
         */
        get: function () {
            return (React.createElement(list_1.SimpleList, { className: this.uiFactory.list }, util_1.subArray(this.menuItems, this.props.maxCount).map(this.getItemElement)));
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [23.09.2019]
     * @param {IMenuItemEntity} option
     * @returns {JSX.Element}
     */
    Menu.prototype.getItemElement = function (option) {
        var props = this.props;
        return (React.createElement(list_1.ListItem, { key: "menu-item-key-" + option.value, disabled: option.disabled, icon: option.icon, rawData: option, renderer: props.renderer, tpl: props.tpl, onClick: this.onSelect }, this.optionValueFn(option)));
    };
    Object.defineProperty(Menu.prototype, "dialog", {
        /**
         * @stable [17.06.2019]
         * @returns {Dialog<AnyT>}
         */
        get: function () {
            return this.dialogRef.current;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Menu.prototype, "field", {
        /**
         * @stable [07.06.2018]
         * @returns {IField}
         */
        get: function () {
            return this.fieldRef.current;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Menu.prototype, "doesDialogExist", {
        /**
         * @stable [17.05.2019]
         * @returns {boolean}
         */
        get: function () {
            return !R.isNil(this.dialog);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Menu.prototype, "filterElement", {
        /**
         * @stable [17.06.2019]
         * @returns {JSX.Element}
         */
        get: function () {
            var props = this.props;
            var state = this.state;
            return (React.createElement(field_1.TextField, { ref: this.fieldRef, full: false, value: state.filter, visible: !!props.useFilter, placeholder: props.filterPlaceholder || this.settings.messages.filterPlaceholderMessage, errorMessageRendered: false, onChange: this.onInputChange }));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Menu.prototype, "closeActionElement", {
        /**
         * @stable [06.07.2019]
         * @returns {JSX.Element}
         */
        get: function () {
            return (React.createElement(layout_1.FlexLayout, { full: false, alignItemsEnd: true, noShrink: true }, this.uiFactory.makeIcon({
                key: 'menu-close-action-key',
                type: 'close',
                className: 'rac-menu-dialog-icon-close',
                onClick: this.onCloseAction,
            })));
        },
        enumerable: true,
        configurable: true
    });
    Menu.defaultProps = {
        filter: function (query, entity) { return util_1.queryFilter(query, entity.label || entity.value); },
    };
    return Menu;
}(base_1.BaseComponent));
exports.Menu = Menu;
//# sourceMappingURL=menu.component.js.map