import * as React from 'react';
import { Store } from 'redux';
import { ErrorEventCategoriesEnum, IDomAccessor, IEnvironment, ILogManager, IRouter, IRoutesEntity, IUiFactory, IUniversalStoreEntity } from '../../../definition';
import { ISettingsEntity } from '../../../settings';
export declare class UIDefaultFactory implements IUiFactory {
    private static readonly WIN_ERROR_ID;
    protected readonly domAccessor: IDomAccessor;
    protected readonly environment: IEnvironment;
    protected readonly logManager: ILogManager;
    protected readonly router: IRouter;
    protected readonly routes: IRoutesEntity;
    protected readonly settings: ISettingsEntity;
    protected readonly store: Store<IUniversalStoreEntity>;
    /**
     * @stable [07.10.2019]
     */
    constructor();
    /**
     * @stable [16.10.2019]
     * @param {Error} e
     * @returns {JSX.Element}
     */
    makeWindowErrorElement(e: Error): Element;
    /**
     * @stable [07.10.2019]
     * @param {Error} e
     * @returns {React.ReactNode}
     */
    makeReactErrorElement(e: Error): React.ReactNode;
    /**
     * @stable [16.10.2019]
     * @param {Error} e
     * @param {Element} parent
     */
    protected makeWindowErrorBodyElement(e: Error, parent: Element): void;
    /**
     * @stable [07.10.2019]
     * @param {Error} e
     * @returns {JSX.Element}
     */
    protected makeReactErrorBodyElement(e: Error): JSX.Element;
    /**
     * @stable [07.10.2019]
     * @param {Error} error
     * @returns {string[]}
     */
    protected buildErrorMessages(error: Error): string[];
    /**
     * @stable [07.10.2019]
     * @returns {string}
     */
    protected readonly errorSystemInfoLine: string;
    /**
     * @stable [07.10.2019]
     * @param {Error} e
     * @returns {JSX.Element}
     */
    protected getErrorMessagesElement(e: Error): JSX.Element;
    /**
     * @stable [07.10.2019]
     * @returns {string[]}
     */
    protected getErrorWrapperClassNames(): string[];
    /**
     * @stable [07.10.2019]
     * @returns {string[]}
     */
    protected getErrorClassNames(): string[];
    /**
     * @stable [16.10.2019]
     * @returns {JSX.Element}
     */
    protected makeReactRestartActionElement(): JSX.Element;
    /**
     * @stable [16.10.2019]
     * @param {Element} parent
     * @returns {Element}
     */
    protected makeWindowRestartActionElement(parent: Element): Element;
    /**
     * @stable [07.10.2019]
     * @returns {string[]}
     */
    protected getRestartActionClassName(): string[];
    /**
     * @stable [07.10.2019]
     */
    protected onRestart(): void;
    /**
     * In case of an out memory error, it would be better to reload the page
     * @stable [16.10.2019]
     */
    protected onRestartAndReload(): void;
    /**
     * @stable [07.10.2019]
     * @param {ErrorEventCategoriesEnum} errorCategory
     * @param {Error} e
     */
    protected logError(errorCategory: ErrorEventCategoriesEnum, e: Error): void;
}
