"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var R = require("ramda");
var inversify_1 = require("inversify");
var di_1 = require("../../../di");
var definition_1 = require("../../../definition");
var util_1 = require("../../../util");
var UIDefaultFactory = /** @class */ (function () {
    /**
     * @stable [07.10.2019]
     */
    function UIDefaultFactory() {
        this.onRestart = this.onRestart.bind(this);
        this.onRestartAndReload = this.onRestartAndReload.bind(this);
    }
    UIDefaultFactory_1 = UIDefaultFactory;
    /**
     * @stable [16.10.2019]
     * @param {Error} e
     * @returns {JSX.Element}
     */
    UIDefaultFactory.prototype.makeWindowErrorElement = function (e) {
        var _a, _b;
        this.logError(definition_1.ErrorEventCategoriesEnum.WINDOW_ERROR, e);
        var el = this.domAccessor.getElement(UIDefaultFactory_1.WIN_ERROR_ID);
        var errorMessageWrapperEl = el || this.domAccessor.createElement();
        if (!R.isNil(el)) {
            // Do nothing, because only one error is shown
        }
        else {
            var errorMessageEl = this.domAccessor.createElement('div', errorMessageWrapperEl);
            errorMessageEl.id = UIDefaultFactory_1.WIN_ERROR_ID;
            (_a = this.domAccessor).addClassNames.apply(_a, [errorMessageWrapperEl].concat(this.getErrorWrapperClassNames()));
            (_b = this.domAccessor).addClassNames.apply(_b, [errorMessageEl].concat(this.getErrorClassNames()));
            this.domAccessor.addChild(errorMessageWrapperEl);
            this.makeWindowErrorBodyElement(e, errorMessageEl);
        }
        return errorMessageWrapperEl;
    };
    /**
     * @stable [07.10.2019]
     * @param {Error} e
     * @returns {React.ReactNode}
     */
    UIDefaultFactory.prototype.makeReactErrorElement = function (e) {
        this.logError(definition_1.ErrorEventCategoriesEnum.REACT_ERROR, e);
        return (React.createElement("div", { className: util_1.joinClassName.apply(void 0, this.getErrorWrapperClassNames()) },
            React.createElement("div", { className: util_1.joinClassName.apply(void 0, this.getErrorClassNames()) }, this.makeReactErrorBodyElement(e))));
    };
    /**
     * @stable [16.10.2019]
     * @param {Error} e
     * @param {Element} parent
     */
    UIDefaultFactory.prototype.makeWindowErrorBodyElement = function (e, parent) {
        this.makeWindowRestartActionElement(parent);
        var content = this.domAccessor.createElement('div', parent);
        content.textContent = this.buildErrorMessages(e).join('<br>');
    };
    /**
     * @stable [07.10.2019]
     * @param {Error} e
     * @returns {JSX.Element}
     */
    UIDefaultFactory.prototype.makeReactErrorBodyElement = function (e) {
        return (React.createElement(React.Fragment, null,
            this.makeReactRestartActionElement(),
            this.getErrorMessagesElement(e)));
    };
    /**
     * @stable [07.10.2019]
     * @param {Error} error
     * @returns {string[]}
     */
    UIDefaultFactory.prototype.buildErrorMessages = function (error) {
        var messages = this.settings.messages;
        return [
            messages.SOMETHING_WENT_WRONG,
            messages.PLS_SEND_THIS_SCR_TO_SUPPORT_MANAGER,
            this.errorSystemInfoLine,
            messages.ERROR + ": " + error.message,
            messages.DETAILS_INFO + ": [" + error.stack + "]"
        ];
    };
    Object.defineProperty(UIDefaultFactory.prototype, "errorSystemInfoLine", {
        /**
         * @stable [07.10.2019]
         * @returns {string}
         */
        get: function () {
            var environment = this.environment;
            var messages = this.settings.messages;
            var user = this.store.getState().user;
            return [
                messages.ENVIRONMENT + ": ",
                [
                    util_1.ifNotNilThanValue(user, function () { return messages.USER + " " + user.id; }),
                    messages.BUILD + " " + environment.appVersion,
                    environment.browserName + " " + environment.browserVersion + environment.platformType,
                    messages.PATH + " " + util_1.getCurrentUrlPath()
                ].join(', ')
            ].join('');
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [07.10.2019]
     * @param {Error} e
     * @returns {JSX.Element}
     */
    UIDefaultFactory.prototype.getErrorMessagesElement = function (e) {
        return (React.createElement(React.Fragment, null, this.buildErrorMessages(e).map(function (v, index) { return React.createElement("div", { key: "error-" + index }, v); })));
    };
    /**
     * @stable [07.10.2019]
     * @returns {string[]}
     */
    UIDefaultFactory.prototype.getErrorWrapperClassNames = function () {
        return ['rac-window-error-wrapper', 'rac-full-size', 'rac-fixed'];
    };
    /**
     * @stable [07.10.2019]
     * @returns {string[]}
     */
    UIDefaultFactory.prototype.getErrorClassNames = function () {
        return ['rac-window-error', 'rac-alignment-center'];
    };
    /**
     * @stable [16.10.2019]
     * @returns {JSX.Element}
     */
    UIDefaultFactory.prototype.makeReactRestartActionElement = function () {
        return (React.createElement("button", __assign({}, util_1.handlerPropsFactory(this.onRestartAndReload), { className: util_1.joinClassName.apply(void 0, this.getRestartActionClassName()) }), this.settings.messages.RESTART_APP));
    };
    /**
     * @stable [16.10.2019]
     * @param {Element} parent
     * @returns {Element}
     */
    UIDefaultFactory.prototype.makeWindowRestartActionElement = function (parent) {
        var _a;
        var actionEl = this.domAccessor.createElement('button', parent);
        (_a = this.domAccessor).addClassNames.apply(_a, [actionEl].concat(this.getRestartActionClassName()));
        actionEl.textContent = this.settings.messages.RESTART_APP;
        actionEl.onclick = this.onRestartAndReload;
        return actionEl;
    };
    /**
     * @stable [07.10.2019]
     * @returns {string[]}
     */
    UIDefaultFactory.prototype.getRestartActionClassName = function () {
        return ['rac-window-error-restart-action'];
    };
    /**
     * @stable [07.10.2019]
     */
    UIDefaultFactory.prototype.onRestart = function () {
        this.router.go(-this.router.length);
        this.router.push(this.routes.logout);
    };
    /**
     * In case of an out memory error, it would be better to reload the page
     * @stable [16.10.2019]
     */
    UIDefaultFactory.prototype.onRestartAndReload = function () {
        this.onRestart();
        this.domAccessor.reload(true);
    };
    /**
     * @stable [07.10.2019]
     * @param {ErrorEventCategoriesEnum} errorCategory
     * @param {Error} e
     */
    UIDefaultFactory.prototype.logError = function (errorCategory, e) {
        this.logManager.send(errorCategory, e.name, e.stack || e.message);
    };
    var UIDefaultFactory_1;
    UIDefaultFactory.WIN_ERROR_ID = '$$windowErrorElement';
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.DomAccessor),
        __metadata("design:type", Object)
    ], UIDefaultFactory.prototype, "domAccessor", void 0);
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.Environment),
        __metadata("design:type", Object)
    ], UIDefaultFactory.prototype, "environment", void 0);
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.LogManager),
        __metadata("design:type", Object)
    ], UIDefaultFactory.prototype, "logManager", void 0);
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.Router),
        __metadata("design:type", Object)
    ], UIDefaultFactory.prototype, "router", void 0);
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.Routes),
        __metadata("design:type", Object)
    ], UIDefaultFactory.prototype, "routes", void 0);
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.Settings),
        __metadata("design:type", Object)
    ], UIDefaultFactory.prototype, "settings", void 0);
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.Store),
        __metadata("design:type", Object)
    ], UIDefaultFactory.prototype, "store", void 0);
    UIDefaultFactory = UIDefaultFactory_1 = __decorate([
        inversify_1.injectable(),
        __metadata("design:paramtypes", [])
    ], UIDefaultFactory);
    return UIDefaultFactory;
}());
exports.UIDefaultFactory = UIDefaultFactory;
//# sourceMappingURL=ui-default-factory.service.js.map