"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var inversify_1 = require("inversify");
var R = require("ramda");
var moment = require("moment");
require("moment-timezone");
var di_1 = require("../../di");
var definitions_interface_1 = require("../../definitions.interface");
var util_1 = require("../../util");
var settings_1 = require("../../settings");
var APP_STARTING_DATE = Date.now();
var DateConverter = /** @class */ (function () {
    function DateConverter() {
    }
    DateConverter_1 = DateConverter;
    /**
     * @stable [16.10.2019]
     * @returns {number}
     */
    DateConverter.prototype.getAppOnlineLifeTimeInSeconds = function () {
        return Math.round(Date.now() - APP_STARTING_DATE) / (1000 * 60);
    };
    /**
     * @stable [16.10.2019]
     * @returns {number}
     */
    DateConverter.prototype.getAppOnlineLifeTimeInHours = function () {
        return Math.round(this.getAppOnlineLifeTimeInSeconds() / 60);
    };
    /**
     * @stable [08.08.2019]
     * @param {DateTimeLikeTypeT} date1
     * @param {DateTimeLikeTypeT} date2
     * @returns {number}
     */
    DateConverter.prototype.compare = function (date1, date2) {
        var date1$ = date1;
        var date2$ = date2;
        return R.equals(date1, date2)
            ? 0
            : (date1$ instanceof Date && date2$ instanceof Date
                ? util_1.NUMBER_COMPARATOR(date1$.getTime(), date2$.getTime())
                : NaN);
    };
    /**
     * @stable [11.08.2018]
     * @param {DateTimeLikeTypeT} date
     * @param {string} inputFormat
     * @param {string} outputFormat
     * @returns {string}
     */
    DateConverter.prototype.format = function (date, inputFormat, outputFormat) {
        if (R.isNil(date) || R.isEmpty(date)) {
            return '';
        }
        else {
            var momentDate = this.toMomentDate(date, inputFormat);
            return momentDate.isValid()
                ? momentDate.format(outputFormat)
                : String(date);
        }
    };
    /**
     * @stable [09.11.2018]
     * @param {DateTimeLikeTypeT} date
     * @param {string} outputFormat
     * @returns {string}
     */
    DateConverter.prototype.fromDateTimeToArbitraryFormat = function (date, outputFormat) {
        return this.format(date, this.dateTimeFormat, outputFormat);
    };
    /**
     * @stable [04.03.2019]
     * @param {DateTimeLikeTypeT} date
     * @param {string} outputFormat
     * @returns {string}
     */
    DateConverter.prototype.fromDateToArbitraryFormat = function (date, outputFormat) {
        return this.format(date, this.dateFormat, outputFormat);
    };
    /**
     * @stable [04.03.2019]
     * @param {DateTimeLikeTypeT} date
     * @returns {number}
     */
    DateConverter.prototype.fromDateToYear = function (date) {
        if (date === void 0) { date = this.currentDate; }
        return this.toYear(date, this.dateFormat);
    };
    /**
     * @stable [09.11.2018]
     * @param {DateTimeLikeTypeT} date [Example: 2018-04-07T20:54:45+03:00]
     * @returns {string} [Example: Apr 07 08:54 PM]
     */
    DateConverter.prototype.fromDateTimeToPstDateTime = function (date) {
        if (date === void 0) { date = this.currentDate; }
        return this.fromDateTimeToArbitraryFormat(date, this.pstDateTimeFormat);
    };
    /**
     * @stable [09.11.2018]
     * @param {DateTimeLikeTypeT} date [Example: 2018-11-30T03:00:00+03:00]
     * @returns {string} [Example: Nov 30]
     */
    DateConverter.prototype.fromDateTimeToPstDate = function (date) {
        if (date === void 0) { date = this.currentDate; }
        return this.fromDateTimeToArbitraryFormat(date, this.dateTimeSettings.pstDateFormat);
    };
    /**
     * @stable [08.01.2019]
     * @param {DateTimeLikeTypeT} date
     * @returns {string}
     */
    DateConverter.prototype.fromDateTimeToPstTime = function (date) {
        if (date === void 0) { date = this.currentDate; }
        return this.fromDateTimeToArbitraryFormat(date, this.dateTimeSettings.pstTimeFormat);
    };
    /**
     * @stable [09.11.2018]
     * @param {DateTimeLikeTypeT} date [Example: 2018-04-07T20:54:45+03:00]
     * @returns {string} [Example: 2018-04-07]
     */
    DateConverter.prototype.fromDateTimeToDate = function (date) {
        return this.fromDateTimeToArbitraryFormat(date, this.dateFormat);
    };
    /**
     * @stable [04.10.2019]
     * @param {DateTimeLikeTypeT} date
     * @returns {string}
     */
    DateConverter.prototype.fromDateTimeToUiDate = function (date) {
        return this.fromDateTimeToArbitraryFormat(date, this.uiDateFormat);
    };
    /**
     * @stable [09.11.2018]
     * @param {DateTimeLikeTypeT} date [Example: 2018-04-07T20:54:45+03:00]
     * @returns {string} [Example: 20:54:45]
     */
    DateConverter.prototype.fromDateTimeToTime = function (date) {
        return this.fromDateTimeToArbitraryFormat(date, this.timeFormat);
    };
    /**
     * @stable [23.11.2018]
     * @param {DateTimeLikeTypeT} date [Example: Wed Oct 24 2018 04:07:41 GMT+0300 (RTZ 2 (зима))]
     * @returns {string} [Example: 2018-10-24T04:07:41+03:00]
     */
    DateConverter.prototype.fromDateTimeToDateTime = function (date) {
        return this.fromDateTimeToArbitraryFormat(date, this.dateTimeFormat);
    };
    /**
     * @stable [24.11.2018]
     * @param {string} date [Example: 2018-11-24]
     * @param {string} time [Example: 03:47:17]
     * @returns {string} [Example: 2018-11-24T03:47:17+03:00]
     */
    DateConverter.prototype.fromUiDateTimeToDateTime = function (date, time) {
        return this.format(this.combine(date, time), this.uiDateTimeFormat, this.dateTimeFormat);
    };
    /**
     * @stable [11.08.2019]
     * @param {DateTimeLikeTypeT} date
     * @returns {string}
     */
    DateConverter.prototype.fromUiDateToDateTime = function (date) {
        return this.format(date, this.uiDateFormat, this.dateTimeFormat);
    };
    /**
     * @stable [11.11.2019]
     * @param {DateTimeLikeTypeT} date
     * @returns {string}
     */
    DateConverter.prototype.fromUiDateToDate = function (date) {
        return this.format(date, this.uiDateFormat, this.dateFormat);
    };
    /**
     * @stable [24.11.2018]
     * @param {string} date [Example: 2018-11-24]
     * @param {string} time [Example: 03:47:17]
     * @returns {string} [Example: Nov 24]
     */
    DateConverter.prototype.fromUiDateTimeToPstDate = function (date, time) {
        return this.fromDateTimeToPstDate(this.fromUiDateTimeToDateTime(date, time));
    };
    /**
     * @stable [09.11.2018]
     * @param {TEntity} entity
     * @param {string} dateFieldName
     * @param {string} timeFieldName
     * @param {(entity: TEntity) => string} dateResolver
     * @returns {IKeyValue}
     */
    DateConverter.prototype.splitToDateTimeFields = function (entity, dateFieldName, timeFieldName, dateResolver) {
        var _this = this;
        var _a;
        var date = dateResolver(entity);
        return util_1.defValuesFilter((_a = {},
            _a[timeFieldName] = util_1.orUndef(date, function () { return _this.fromDateTimeToTime(date); }),
            _a[dateFieldName] = util_1.orUndef(date, function () { return _this.fromDateTimeToDate(date); }),
            _a));
    };
    /**
     * @stable [07.01.2019]
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {DateTimeLikeTypeT}
     */
    DateConverter.prototype.tryConvertToDate = function (date, inputFormat) {
        if (inputFormat === void 0) { inputFormat = this.dateFormat; }
        var momentDate = this.toMomentDate(date, inputFormat);
        return momentDate.isValid() ? momentDate.toDate() : date;
    };
    /**
     * @stable [06.06.2019]
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {number}
     */
    DateConverter.prototype.tryConvertToDateAsTime = function (date, inputFormat) {
        if (inputFormat === void 0) { inputFormat = this.dateTimeFormat; }
        var momentDate = this.toMomentDate(date, inputFormat);
        return momentDate.isValid() ? momentDate.toDate().getTime() : -1;
    };
    /**
     * @stable [07.01.2019]
     * @param {moment.DurationInputArg2} unit
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {moment.Moment}
     */
    DateConverter.prototype.tryAddXDurationAsMomentDate = function (unit, duration, date, inputFormat) {
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        var momentDate = this.toMomentDate(date, inputFormat);
        return util_1.orNull(momentDate.isValid(), function () { return momentDate.add(duration, unit); });
    };
    /**
     * @stable [07.01.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {moment.Moment}
     */
    DateConverter.prototype.tryAddXDaysAsMomentDate = function (duration, date, inputFormat) {
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        return this.tryAddXDurationAsMomentDate('days', duration, date, inputFormat);
    };
    /**
     * @stable [11.08.2019]
     * @param {DateTimeLikeTypeT} date
     * @param {string} inputFormat
     * @returns {number}
     */
    DateConverter.prototype.tryGetWeekdayNumber = function (date, inputFormat) {
        var _this = this;
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        var momentDate = this.toMomentDate(date, inputFormat);
        return util_1.orNull(momentDate.isValid(), function () { return _this.settings.dateTime.startDayOfWeek === settings_1.StartDayOfWeekT.MONDAY // TODO make getter
            ? momentDate.isoWeekday() : momentDate.weekday(); });
    };
    /**
     * @stable [07.01.2019]
     * @param {moment.DurationInputArg2} unit
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.tryAddXDuration = function (unit, duration, date, inputFormat) {
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        return util_1.ifNotNilThanValue(this.tryAddXDurationAsMomentDate(unit, duration, date, inputFormat), function (value) { return value.toDate(); });
    };
    /**
     * @stable [07.01.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.tryAddXDays = function (duration, date, inputFormat) {
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        return this.tryAddXDuration('days', duration, date, inputFormat);
    };
    /**
     * @stable [10.02.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.tryAddXWeeks = function (duration, date, inputFormat) {
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        return this.tryAddXDuration('weeks', duration, date, inputFormat);
    };
    /**
     * @stable [07.01.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.tryAddXMonths = function (duration, date, inputFormat) {
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        return this.tryAddXDuration('months', duration, date, inputFormat);
    };
    /**
     * @stable [10.02.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.tryAddXQuarters = function (duration, date, inputFormat) {
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        return this.tryAddXDuration('quarters', duration, date, inputFormat);
    };
    /**
     * @stable [10.02.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.tryAddXYears = function (duration, date, inputFormat) {
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        return this.tryAddXDuration('years', duration, date, inputFormat);
    };
    /**
     * @stable [09.02.2019]
     * @param {moment.DurationInputArg2} unit
     * @param {moment.unitOfTime.StartOf} startOf
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {moment.Moment}
     */
    DateConverter.prototype.tryGetFirstDayOfXAsMomentDate = function (unit, startOf, duration, date, inputFormat) {
        if (duration === void 0) { duration = 0; }
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        return util_1.ifNotNilThanValue(this.tryAddXDurationAsMomentDate(unit, duration, date, inputFormat), function (value) { return value.startOf(startOf); });
    };
    /**
     * @stable [08.01.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {moment.Moment}
     */
    DateConverter.prototype.tryGetFirstDayOfMonthAsMomentDate = function (duration, date, inputFormat) {
        if (duration === void 0) { duration = 0; }
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        return this.tryGetFirstDayOfXAsMomentDate('months', 'month', duration, date, inputFormat);
    };
    /**
     * @stable [10.01.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {moment.Moment}
     */
    DateConverter.prototype.tryGetFirstDayOfYearAsMomentDate = function (duration, date, inputFormat) {
        if (duration === void 0) { duration = 0; }
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        return this.tryGetFirstDayOfXAsMomentDate('years', 'year', duration, date, inputFormat);
    };
    /**
     * @stable [10.02.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {moment.Moment}
     */
    DateConverter.prototype.tryGetFirstDayOfQuarterAsMomentDate = function (duration, date, inputFormat) {
        if (duration === void 0) { duration = 0; }
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        return this.tryGetFirstDayOfXAsMomentDate('quarters', 'quarter', duration, date, inputFormat);
    };
    /**
     * @stable [09.02.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {moment.Moment}
     */
    DateConverter.prototype.tryGetFirstDayOfWeekAsMomentDate = function (duration, date, inputFormat) {
        if (duration === void 0) { duration = 0; }
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        return this.tryGetFirstDayOfXAsMomentDate('weeks', this.settings.dateTime.startDayOfWeek === settings_1.StartDayOfWeekT.MONDAY ? 'isoWeek' : 'week', duration, date, inputFormat);
    };
    /**
     * @stable [09.02.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.tryGetFirstDayOfWeek = function (duration, date, inputFormat) {
        if (duration === void 0) { duration = 0; }
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        return util_1.ifNotNilThanValue(this.tryGetFirstDayOfWeekAsMomentDate(duration, date, inputFormat), function (value) { return value.toDate(); });
    };
    /**
     * @stable [08.01.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.tryGetFirstDayOfMonth = function (duration, date, inputFormat) {
        if (duration === void 0) { duration = 0; }
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        return util_1.ifNotNilThanValue(this.tryGetFirstDayOfMonthAsMomentDate(duration, date, inputFormat), function (value) { return value.toDate(); });
    };
    /**
     * @stable [10.02.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.tryGetFirstDayOfQuarter = function (duration, date, inputFormat) {
        if (duration === void 0) { duration = 0; }
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        return util_1.ifNotNilThanValue(this.tryGetFirstDayOfQuarterAsMomentDate(duration, date, inputFormat), function (value) { return value.toDate(); });
    };
    /**
     * @stable [10.02.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.tryGetFirstDayOfYear = function (duration, date, inputFormat) {
        if (duration === void 0) { duration = 0; }
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.uiDateFormat; }
        return util_1.ifNotNilThanValue(this.tryGetFirstDayOfYearAsMomentDate(duration, date, inputFormat), function (value) { return value.toDate(); });
    };
    /**
     * @stable [08.08.2019]
     * @param {DateTimeLikeTypeT} date
     * @param {string} inputFormat
     * @returns {boolean}
     */
    DateConverter.prototype.isDateBelongsToCurrentWeek = function (date, inputFormat) {
        if (inputFormat === void 0) { inputFormat = this.dateFormat; }
        var momentDate = this.toMomentDate(date, inputFormat);
        return momentDate.isValid() && (momentDate.toDate() >= this.tryGetFirstDayOfWeek());
    };
    /**
     * @deprecated Use fromDateTimeToDate
     */
    DateConverter.prototype.fromDateToUiDate = function (date) {
        return this.format(this.shrinkDate(date), /* Need to clear a timezone */ this.dateFormat, this.uiDateFormat);
    };
    DateConverter.prototype.fromDateTimeToUiDateTime = function (date) {
        return this.fromDateTimeToArbitraryFormat(date, this.uiDateTimeFormat);
    };
    /**
     * @test
     * @param {string} startUiDate [2018-01-31]
     * @param {string} startUiTime [21:58:59]
     * @returns {string} The formatted value [2018-01-31T21:58:59-08:00]
     */
    DateConverter.prototype.fromStartUiDateTimeToDateTime = function (startUiDate, startUiTime) {
        if (startUiTime === void 0) { startUiTime = definitions_interface_1.DEFAULT_TIME_FROM; }
        return this.fromUiDateTimeToDateTime(startUiDate, startUiTime);
    };
    /**
     * @test
     * @param {string} endUiDate [2018-01-31]
     * @param {string} endUiTime [23:59:59]
     * @returns {string} The formatted value [2018-01-31T23:59:59-08:00]
     */
    DateConverter.prototype.fromEndUiDateTimeToDateTime = function (endUiDate, endUiTime) {
        if (endUiTime === void 0) { endUiTime = definitions_interface_1.DEFAULT_TIME_TO; }
        return this.fromUiDateTimeToDateTime(endUiDate, endUiTime);
    };
    /**
     * @deprecated
     */
    DateConverter.prototype.from30DaysAgoUiDateTimeToDateTime = function () {
        return this.fromUiDateTimeToDateTime(this.fromDateToUiDate(this.get30DaysAgo()), definitions_interface_1.DEFAULT_TIME_FROM);
    };
    /**
     * @deprecated
     */
    DateConverter.prototype.get30DaysAgo = function () {
        return this.getXDaysAgo(30);
    };
    /**
     * @deprecated
     */
    DateConverter.prototype.getXDaysLater = function (days) {
        return this.getCurrentMomentDate().add(days, 'days').toDate();
    };
    /**
     * @deprecated
     */
    DateConverter.prototype.getXDaysAgo = function (days) {
        return this.getCurrentMomentDate().subtract(days, 'days').toDate();
    };
    /**
     * @deprecated
     */
    DateConverter.prototype.addXDays = function (date, days) {
        return this.toMomentDate(date).add(days, 'days').toDate();
    };
    /**
     * @stable [29.08.2019]
     * @returns {Date}
     */
    DateConverter.prototype.getCurrentDate = function () {
        return this.currentDate;
    };
    /**
     * @deprecated
     */
    DateConverter.prototype.getCurrentTime = function () {
        return new Date();
    };
    /**
     * @stable [08.08.2019]
     * @returns {Date}
     */
    DateConverter.prototype.getStartOfCurrentDate = function () {
        var date = new Date(this.currentDate);
        date.setHours(0, 0, 0, 0);
        return date;
    };
    /**
     * @stable [09.05.2019]
     * @returns {Date}
     */
    DateConverter.prototype.getYesterdayDate = function () {
        return this.tryAddXDays(-1);
    };
    DateConverter.prototype.formatDate = function (date, outputFormat) {
        return this.format(date, this.dateFormat, outputFormat);
    };
    DateConverter.prototype.formatDateTimeFromDateTime = function (date) {
        return this.formatDateTime(date, this.dateTimeFormat);
    };
    DateConverter.prototype.formatTimeFromDateTime = function (date) {
        return this.formatDateTime(date, this.timeFormat);
    };
    DateConverter.prototype.formatDateTime = function (date, outputFormat) {
        return this.format(date, this.dateTimeFormat, outputFormat);
    };
    DateConverter.prototype.formatPSTDate = function (date) {
        if (date === void 0) { date = new Date(); }
        return this.formatDate(date, this.dateTimeSettings.pstDateFormat);
    };
    DateConverter.prototype.appendToDate = function (date, data, inputFormat) {
        if (inputFormat === void 0) { inputFormat = this.dateFormat; }
        var momentDate = this.toMomentDate(date, inputFormat);
        data.forEach(function (item) {
            momentDate = momentDate.add(item[0], item[1]);
        });
        return momentDate.toDate();
    };
    DateConverter.prototype.getLocalizedMonth = function (index) {
        return DateConverter_1.MONTHS[index];
    };
    DateConverter.prototype.getLocalizedMonthShort = function (index) {
        return DateConverter_1.MONTHS_SHORT[index];
    };
    DateConverter.prototype.getLocalizedWeekday = function (index) {
        return DateConverter_1.WEEKDAYS[index];
    };
    DateConverter.prototype.getLocalizedWeekdayShort = function (index) {
        return DateConverter_1.WEEKDAYS_SHORT[index];
    };
    /**
     * @stable [25.08.2018]
     * @param {number} monthsAgo
     * @param {DateTimeLikeTypeT} date
     * @param {string} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.getLastDayOfMonth = function (monthsAgo, date, inputFormat) {
        if (monthsAgo === void 0) { monthsAgo = 1; }
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.dateTimeSettings.uiDateFormat; }
        var momentDate = this.toMomentDate(date, inputFormat);
        return util_1.orNull(momentDate.isValid(), function () { return momentDate.subtract(monthsAgo, 'months').endOf('month').toDate(); });
    };
    /**
     * @stable [25.08.2018]
     * @param {number} monthsAgo
     * @param {DateTimeLikeTypeT} date
     * @param {string} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.getFirstDayOfMonth = function (monthsAgo, date, inputFormat) {
        if (monthsAgo === void 0) { monthsAgo = 0; }
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.dateTimeSettings.uiDateFormat; }
        var momentDate = this.toMomentDate(date, inputFormat);
        return util_1.orNull(momentDate.isValid(), function () { return momentDate.subtract(monthsAgo, 'months').startOf('month').toDate(); });
    };
    DateConverter.prototype.getPersonAge = function (birthday, date, inputFormat) {
        if (date === void 0) { date = this.currentDate; }
        return this.toMomentDate(date, inputFormat).diff(birthday, 'years');
    };
    DateConverter.prototype.isSameMonth = function (date1, date2) {
        return (date1.getMonth() === date2.getMonth() &&
            date1.getFullYear() === date2.getFullYear());
    };
    DateConverter.prototype.isSameDay = function (date1, date2) {
        return this.isSameMonth(date1, date2) && date1.getDate() === date2.getDate();
    };
    DateConverter.prototype.isWeekend = function (day) {
        return day === 0 || day === 6;
    };
    /**
     * @stable [24.11.2018]
     * @param {string} dateAsString
     * @param {string} timeAsString
     * @returns {string}
     */
    DateConverter.prototype.combine = function (dateAsString, timeAsString) {
        return [dateAsString, timeAsString].join(' ');
    };
    /**
     * @stable [25.08.2018]
     * @param {DateTimeLikeTypeT} dateTime
     * @param {string} separator
     * @returns {DateTimeLikeTypeT}
     */
    DateConverter.prototype.shrinkDate = function (dateTime, separator) {
        if (separator === void 0) { separator = 'T'; }
        return util_1.isString(dateTime) ? dateTime.split(separator)[0] : dateTime;
    };
    /**
     * @stable [03.04.2019]
     * @param {DateTimeLikeTypeT} date
     * @param {string} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.toDate = function (date, inputFormat) {
        return this.toMomentDate(date, inputFormat).toDate();
    };
    /**
     * @stable [11.08.2018]
     * @param {DateTimeLikeTypeT} date
     * @param {string} inputFormat
     * @returns {moment.Moment}
     */
    DateConverter.prototype.toMomentDate = function (date, inputFormat) {
        var momentDate = date instanceof Date
            ? moment(date)
            : moment(date, inputFormat, true);
        var zone = this.timeZone;
        return zone ? moment.tz(date, zone) : momentDate;
    };
    /**
     * @deprecated
     */
    DateConverter.prototype.getCurrentMomentDate = function () {
        return this.toMomentDate(this.currentDate);
    };
    Object.defineProperty(DateConverter.prototype, "timeZone", {
        /**
         * @stable [11.08.2018]
         * @returns {string}
         */
        get: function () {
            return this.dateTimeSettings.timeZone;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "currentDate", {
        get: function () {
            return this.dateTimeSettings.currentDate;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "dateTimeFormat", {
        get: function () {
            return this.dateTimeSettings.dateTimeFormat;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "dateFormat", {
        get: function () {
            return this.dateTimeSettings.dateFormat;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "timeFormat", {
        get: function () {
            return this.dateTimeSettings.timeFormat;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "uiDateFormat", {
        get: function () {
            return this.dateTimeSettings.uiDateFormat;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "uiTimeFormat", {
        get: function () {
            return this.dateTimeSettings.uiTimeFormat;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "uiDateTimeFormat", {
        get: function () {
            return this.combine(this.uiDateFormat, this.uiTimeFormat);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "pstDateTimeFormat", {
        get: function () {
            return this.combine(this.dateTimeSettings.pstDateFormat, this.dateTimeSettings.pstTimeFormat);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "dateTimeSettings", {
        get: function () {
            return this.settings.dateTime || {};
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [04.03.2019]
     * @param {DateTimeLikeTypeT} date
     * @param {string} inputFormat
     * @returns {number}
     */
    DateConverter.prototype.toYear = function (date, inputFormat) {
        return this.toDate(date, inputFormat).getFullYear();
    };
    /**
     * @stable [04.03.2019]
     * @param {DateTimeLikeTypeT} date
     * @param {string} inputFormat
     * @returns {number}
     */
    DateConverter.prototype.toMonth = function (date, inputFormat) {
        return this.toDate(date, inputFormat).getMonth();
    };
    var DateConverter_1;
    DateConverter.MONTHS = moment.months();
    DateConverter.MONTHS_SHORT = moment.monthsShort();
    DateConverter.WEEKDAYS_SHORT = moment.weekdaysShort()
        .slice(1, 7)
        .concat(moment.weekdaysShort()[0]);
    DateConverter.WEEKDAYS = moment.weekdays()
        .slice(1, 7)
        .concat(moment.weekdays()[0]);
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.Settings),
        __metadata("design:type", Object)
    ], DateConverter.prototype, "settings", void 0);
    DateConverter = DateConverter_1 = __decorate([
        inversify_1.injectable()
    ], DateConverter);
    return DateConverter;
}());
exports.DateConverter = DateConverter;
//# sourceMappingURL=date-converter.service.js.map