import { IEntity, IKeyValue } from '../definitions.interface';
import { IGenericFieldEntity, IMultiEntity, MultiFieldEntityT, NotMultiFieldEntityT } from '../definition';
/**
 * @stable [30.10.2019]
 * @param {IGenericFieldEntity} config
 * @returns {AnyT}
 */
export declare const buildFinalFieldValue: (config: IGenericFieldEntity) => any;
/**
 * @stable [28.10.2019]
 * @param {IGenericFieldEntity} props
 * @returns {boolean}
 */
export declare const isFieldInactive: (props: IGenericFieldEntity) => boolean;
/**
 * @stable [30.10.2019]
 * @param {IGenericFieldEntity} props
 * @returns {boolean}
 */
export declare const isFieldNotModifiable: (props: IGenericFieldEntity) => boolean;
/**
 * @stable [30.10.2019]
 * @param {AnyT} value
 * @param {AnyT} emptyValue
 * @returns {boolean}
 */
export declare const isFieldValuePresent: (value: any, emptyValue: any) => boolean;
/**
 * @stable [27.08.2019]
 * @param {EntityIdT} key
 * @returns {string}
 */
export declare const dynamicFieldName: (key: string | number) => string;
/**
 * @stable [28.08.2019]
 * @param {TEntity} object
 * @param {EntityIdT} key
 * @returns {AnyT}
 */
export declare const dynamicFieldValue: <TEntity extends IEntity>(object: TEntity, key: string | number) => any;
/**
 * @stable [28.08.2019]
 * @param {TEntity[]} array
 * @param {(itm: TEntity) => EntityIdT} keyAccessor
 * @param {(itm: TEntity) => EntityIdT} valueAccessor
 * @returns {IKeyValue}
 */
export declare const fromDynamicFieldsArray: <TEntity extends string | number | IEntity>(array: TEntity[], keyAccessor: (itm: TEntity) => string | number, valueAccessor: (itm: TEntity) => string | number) => IKeyValue;
/**
 * @stable [28.08.2019]
 * @param {EntityIdT[]} array
 * @param {(itm: EntityIdT) => EntityIdT} valueAccessor
 * @returns {IKeyValue}
 */
export declare const fromDynamicFieldsIdsArray: (array: (string | number)[], valueAccessor: (itm: string | number) => string | number) => IKeyValue;
/**
 * @stable [13.10.2019]
 * @param {MultiFieldEntityT<TEntity extends IEntity>} entity
 * @returns {TEntity[]}
 */
export declare const asMultiFieldEntities: <TEntity extends IEntity = IEntity>(entity: MultiFieldEntityT<TEntity>) => TEntity[];
/**
 * @stable [12.10.2019]
 * @param {MultiFieldEntityT} entity
 * @param {Map<EntityIdT, TItem extends IEntity>} mappedSourcedItems
 * @returns {TItem[]}
 */
export declare const asMultiFieldEditedEntities: <TEntity extends IEntity = IEntity>(entity: MultiFieldEntityT<TEntity>, mappedSourcedItems?: Map<string | number, TEntity>) => TEntity[];
/**
 * @stable [14.10.2019]
 * @param {NotMultiFieldEntityT} value
 * @returns {IEntity[]}
 */
export declare const asEntitiesArray: <TEntity extends IEntity = IEntity>(value: NotMultiFieldEntityT<TEntity>) => TEntity[];
/**
 * @stable [14.10.2019]
 * @param {MultiFieldEntityT | EntityIdT} value
 * @returns {number}
 */
export declare const asMultiFieldEntitiesLength: (value: string | number | IEntity[] | IMultiEntity<IEntity>) => number;
/**
 * @stable [14.10.2019]
 * @param {MultiFieldEntityT<TEntity extends IEntity>} value
 * @param {number} entitiesCountLimit
 * @returns {TEntity[]}
 */
export declare const asOrderedMultiFieldEntities: <TEntity extends IEntity = IEntity>(value: MultiFieldEntityT<TEntity>, entitiesCountLimit: number) => TEntity[];
/**
 * @stable [14.10.2019]
 * @param {MultiFieldEntityT<TEntity extends IEntity>} multiFieldEntity
 * @param {(entity: TEntity, index: number) => TResult} mapper
 * @returns {TResult[]}
 */
export declare const asMultiFieldMappedEntities: <TEntity extends IEntity = IEntity, TResult = TEntity>(multiFieldEntity: (string | number)[] | IMultiEntity<IEntity> | TEntity[], mapper: (entity: TEntity, index: number) => TResult) => TResult[];
/**
 * @stable [14.10.2019]
 * @param {MultiFieldEntityT<TEntity extends IEntity> | EntityIdT[]} multiFieldEntity
 * @returns {EntityIdT[]}
 */
export declare const asMultiFieldMappedEntitiesIds: <TEntity extends IEntity = IEntity, TResult = TEntity>(multiFieldEntity: (string | number)[] | IMultiEntity<IEntity> | TEntity[]) => (string | number)[];
