"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var type_1 = require("./type");
var object_1 = require("./object");
var cond_1 = require("./cond");
var definitions_interface_1 = require("../definitions.interface");
var uuid_1 = require("./uuid");
/**
 * @stable [02.10.2019]
 * @param {TEntity} changes
 * @param {Array<(entity: TEntity) => MultiFieldSingleValueT>} fieldsValuesResolvers
 * @param {(fileName: string, value: MultiFieldSingleValueT) => Promise<IMultiEntityStorageSetEntity>} storageProcessor
 * @returns {Promise<IMultiEntityStorageSetEntity[]>}
 */
exports.entitiesAsStorageTasks = function (changes, fieldsValuesResolvers, storageProcessor) {
    var tasks = [];
    fieldsValuesResolvers.forEach(function (fieldValueResolver, index) {
        var fieldValueChange = fieldValueResolver(changes);
        tasks.push(type_1.isDef(fieldValueChange)
            ? storageProcessor(uuid_1.uuid(), fieldValueChange)
            : Promise.resolve(null));
    });
    return cond_1.orNull(object_1.isObjectNotEmpty(tasks), function () { return Promise.all(tasks); });
};
/**
 * @stable [02.10.2019]
 * @param {IMultiEntityStorageSetEntity} result
 * @returns {string}
 */
exports.asSingleAddedFileId = function (result) {
    return cond_1.ifNotNilThanValue(result, function () { return (cond_1.ifNotNilThanValue(result.addedFiles[0], function (entity) { return entity.id; }, definitions_interface_1.UNDEF_SYMBOL)); }, definitions_interface_1.UNDEF_SYMBOL);
};
//# sourceMappingURL=storage.js.map