"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var di_1 = require("../di");
var DelayedTask = /** @class */ (function () {
    /**
     * @stable [13.10.2019]
     * @param {(context?: AnyT) => void} task
     * @param {number} period
     * @param {boolean} repeat
     */
    function DelayedTask(task, period, repeat) {
        if (period === void 0) { period = 0; }
        if (repeat === void 0) { repeat = false; }
        this.task = task;
        this.period = period;
        this.repeat = repeat;
    }
    Object.defineProperty(DelayedTask.prototype, "progress", {
        /**
         * @stable [13.10.2019]
         * @returns {boolean}
         */
        get: function () {
            return !R.isNil(this.taskId);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [29.07.2018]
     * @param {AnyT} context
     */
    DelayedTask.prototype.start = function (context) {
        this.context = context;
        this.launchTask();
    };
    /**
     * @stable [29.07.2018]
     */
    DelayedTask.prototype.stop = function () {
        if (R.isNil(this.taskId)) {
            return;
        }
        clearTimeout(this.taskId);
        this.taskId = null;
    };
    /**
     * @stable [29.07.2018]
     */
    DelayedTask.prototype.launchTask = function () {
        var _this = this;
        this.stop();
        this.taskId = di_1.getEnvironment().window.setTimeout(function () { return _this.onTaskDone(); }, this.period);
    };
    /**
     * @stable [29.07.2018]
     */
    DelayedTask.prototype.onTaskDone = function () {
        this.task.call(null, this.context);
        if (this.repeat) {
            this.launchTask();
        }
        else {
            this.taskId = null;
            this.context = null;
        }
    };
    return DelayedTask;
}());
exports.DelayedTask = DelayedTask;
//# sourceMappingURL=task.js.map